use std::path::Path;
use std::{fmt::Display, path::PathBuf};

use crate::error::Error;

use super::{Section, SymbolOffsets, SymbolValue};

pub type SectionIterBox<'iter> = Box<dyn Iterator<Item = Result<Section, Error>> + 'iter>;

/// Contains all the needed getters to construct a final
/// mushed and relocated executable from an object file
pub trait Relocatable: Display + Sized {
    // + TryFrom<PathBuf, Error = Error> {
    fn new(origin: PathBuf, object_index: usize) -> Result<Self, Error>;

    fn object_index(&self) -> usize;

    fn origin(&self) -> &Path; // not same as section's path since this one's supposed to be cannonical

    fn sections(&self) -> SectionIterBox;

    fn symbol_count(&self) -> usize;

    fn symbol_name(&self, index: usize) -> Result<&str, Error>;

    fn symbol_value(&self, index: usize) -> Result<SymbolValue, Error>;

    fn symbol_offsets(&self, index: usize) -> Result<SymbolOffsets, Error>;

    fn symbol_section_index(&self, index: usize) -> Result<usize, Error>;

    fn symbol_needs_resolving(&self, index: usize) -> bool;

    fn bytes(&self, offset: u64, size: u64) -> Result<&[u8], Error>;
}
